<?php

namespace Bixo\Photorequest\Http\Controllers;

use Exception;
use Bixo\Photorequest\Actions\PhotorequestWorkflow;
use Bixo\Photorequest\Http\Requests\PhotorequestWorkflowRequest;
use Bixo\Photorequest\Http\Resources\PhotorequestResource;
use Bixo\Photorequest\Models\Photorequest;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class PhotorequestWorkflowController extends BaseController
{
    /**
     * Action controller function for photorequest.
     *
     * @param Model $photorequest
     * @param action next action for the photorequest.
     *
     * @return Response
     */
    public function __invoke(PhotorequestWorkflowRequest $request, Photorequest $photorequest, $trasition)
    {
        try {
            $request = $request->all();
            $photorequest = PhotorequestWorkflow::run($trasition, $photorequest, $request);
            $data = new PhotorequestResource($photorequest);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('photorequest::photorequest.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('photorequest/photorequest/' . $photorequest->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('photorequest/photorequest/' . $photorequest->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the photorequest and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $photorequest
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $photorequest, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $photorequest = Photorequest::findBySignedId($photorequest);
        $transitions = $this->transitions($photorequest, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('photorequest::photorequest.name') . ' ' . trans('Approvals'))
            ->view('photorequest::public.photorequest.workflow')
            ->layout('mobile')
            ->data(compact('photorequest', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the photorequest.
     *
     * @param PublicRequest $request
     * @param string $photorequest
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $photorequest, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $photorequest = Photorequest::findBySignedId($photorequest);
        $user = user();
        $photorequest = PhotorequestWorkflow::run($transition, $photorequest, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($photorequest, $user)
    {
        $transitions = [];
        foreach ($photorequest->workflow()->transitions($photorequest) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.photorequest.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'photorequest' => $photorequest->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $photorequest->workflow()->form($value);
            $array['label'] = trans('photorequest::photorequest.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
